@@ -15,9 +15,12 @@
 #include <QDateTime>
 #include <QSettings>
 #include <QStandardPaths>
+#include <QTimeZone>
 
 #include <math.h>
 
+using namespace Qt::Literals;
+
 KFormatPrivate::KFormatPrivate(const QLocale &locale)
     : m_locale(locale)
 {
@@ -306,15 +309,15 @@ QString KFormatPrivate::formatByteSize(double size, int precision, KFormat::Bina
     return numString;
 }
 
-enum TimeConstants {
-    MSecsInDay = 86400000,
-    MSecsInHour = 3600000,
-    MSecsInMinute = 60000,
-    MSecsInSecond = 1000,
-};
+constexpr quint64 MSecsInYear = (quint64)1000 * 60 * 60 * 24 * 365; // approximation of a year
+constexpr quint64 MSecsInDay = 1000 * 60 * 60 * 24;
+constexpr quint64 MSecsInHour = 3600000;
+constexpr quint64 MSecsInMinute = 60000;
+constexpr quint64 MSecsInSecond = 1000;
 
 enum DurationUnits {
-    Days = 0,
+    Years = 0,
+    Days,
     Hours,
     Minutes,
     Seconds,
@@ -323,6 +326,11 @@ enum DurationUnits {
 static QString formatSingleAbbreviatedDuration(DurationUnits units, int n)
 {
     switch (units) {
+    case Years:
+        //: @item:intext abbreviated amount of years
+        //~ singular %n yr
+        //~ plural %n yr
+        return KFormatPrivate::tr("%n yr", nullptr, n);
     case Days:
         //: @item:intext abbreviated amount of days
         //~ singular %n d
@@ -358,14 +366,14 @@ QString KFormatPrivate::formatDuration(quint64 msecs, KFormat::DurationFormatOpt
         ms = qRound64(ms / (qreal)MSecsInSecond) * MSecsInSecond;
     }
 
-    int hours = ms / MSecsInHour;
-    ms = ms % MSecsInHour;
-    int minutes = ms / MSecsInMinute;
-    ms = ms % MSecsInMinute;
-    int seconds = ms / MSecsInSecond;
-    ms = ms % MSecsInSecond;
-
     if ((options & KFormat::InitialDuration) == KFormat::InitialDuration) {
+        int hours = ms / MSecsInHour;
+        ms = ms % MSecsInHour;
+        int minutes = ms / MSecsInMinute;
+        ms = ms % MSecsInMinute;
+        int seconds = ms / MSecsInSecond;
+        ms = ms % MSecsInSecond;
+
         if ((options & KFormat::FoldHours) == KFormat::FoldHours && (options & KFormat::ShowMilliseconds) == KFormat::ShowMilliseconds) {
             //: @item:intext Duration format minutes, seconds and milliseconds
             return tr("%1m%2.%3s").arg(hours * 60 + minutes, 1, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0')).arg(ms, 3, 10, QLatin1Char('0'));
@@ -382,65 +390,90 @@ QString KFormatPrivate::formatDuration(quint64 msecs, KFormat::DurationFormatOpt
                 .arg(minutes, 2, 10, QLatin1Char('0'))
                 .arg(seconds, 2, 10, QLatin1Char('0'))
                 .arg(ms, 3, 10, QLatin1Char('0'));
-        } else { // Default
-            //: @item:intext Duration format hours, minutes, seconds
-            return tr("%1h%2m%3s").arg(hours, 1, 10, QLatin1Char('0')).arg(minutes, 2, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0'));
         }
 
-    } else if (options & KFormat::AbbreviatedDuration) {
+        //: @item:intext Duration format hours, minutes, seconds
+        return tr("%1h%2m%3s").arg(hours, 1, 10, QLatin1Char('0')).arg(minutes, 2, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0'));
+    }
+
+    int days = 0;
+    int years = 0;
+    if (options & KFormat::AbbreviatedDuration) {
+        years = ms / MSecsInYear;
+        ms = ms % MSecsInYear;
+        days = ms / MSecsInDay;
+        ms = ms % MSecsInDay;
+    }
+    int hours = ms / MSecsInHour;
+    ms = ms % MSecsInHour;
+    int minutes = ms / MSecsInMinute;
+    ms = ms % MSecsInMinute;
+    int seconds = ms / MSecsInSecond;
+    ms = ms % MSecsInSecond;
+
+    if (options & KFormat::AbbreviatedDuration) {
         if (options & KFormat::FoldHours) {
-            minutes += 60 * hours;
+            minutes += 60 * hours + 60 * 24 * days + 365 * 60 * 24 * years;
             hours = 0;
+            days = 0;
+            years = 0;
         }
 
-        if (hours == 0 && minutes == 0) {
+        if (days == 0 && hours == 0 && minutes == 0) {
             return (options & KFormat::HideSeconds) ? formatSingleAbbreviatedDuration(Minutes, minutes) : formatSingleAbbreviatedDuration(Seconds, seconds);
         }
 
-        if (hours == 0) {
-            if (options & KFormat::HideSeconds) {
-                return formatSingleAbbreviatedDuration(Minutes, minutes);
+        if (years != 0) {
+            if (days == 0) {
+                return formatSingleAbbreviatedDuration(Years, years);
             }
-            //: @item:intext abbreviated amount of minutes and abbreviated amount of seconds
-            return tr("%1 %2").arg(formatSingleAbbreviatedDuration(Minutes, minutes), formatSingleAbbreviatedDuration(Seconds, seconds));
+
+            //: @item:intext abbreviated amount of years and abbreviated amount of days
+            return tr("%1 %2").arg(formatSingleAbbreviatedDuration(Years, years), formatSingleAbbreviatedDuration(Days, days));
         }
 
-        if (options & KFormat::HideSeconds) {
+        if (days != 0) {
+            if (hours == 0) {
+                return formatSingleAbbreviatedDuration(Days, days);
+            }
+
+            //: @item:intext abbreviated amount of days and abbreviated amount of hours
+            return tr("%1 %2").arg(formatSingleAbbreviatedDuration(Days, days), formatSingleAbbreviatedDuration(Hours, hours));
+        }
+
+        if (hours != 0) {
             //: @item:intext abbreviated amount of hours and abbreviated amount of minutes
             return tr("%1 %2").arg(formatSingleAbbreviatedDuration(Hours, hours), formatSingleAbbreviatedDuration(Minutes, minutes));
         }
 
-        //: @item:intext abbreviated amount of hours, abbreviated amount of minutes and abbreviated amount of seconds
-        return tr("%1 %2 %3")
-            .arg(formatSingleAbbreviatedDuration(Hours, hours),
-                 formatSingleAbbreviatedDuration(Minutes, minutes),
-                 formatSingleAbbreviatedDuration(Seconds, seconds));
-
-    } else {
-        if ((options & KFormat::FoldHours) == KFormat::FoldHours && (options & KFormat::ShowMilliseconds) == KFormat::ShowMilliseconds) {
-            //: @item:intext Duration format minutes, seconds and milliseconds
-            return tr("%1:%2.%3").arg(hours * 60 + minutes, 1, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0')).arg(ms, 3, 10, QLatin1Char('0'));
-        } else if ((options & KFormat::FoldHours) == KFormat::FoldHours) {
-            //: @item:intext Duration format minutes and seconds
-            return tr("%1:%2").arg(hours * 60 + minutes, 1, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0'));
-        } else if ((options & KFormat::HideSeconds) == KFormat::HideSeconds) {
-            //: @item:intext Duration format hours and minutes
-            return tr("%1:%2").arg(hours, 1, 10, QLatin1Char('0')).arg(minutes, 2, 10, QLatin1Char('0'));
-        } else if ((options & KFormat::ShowMilliseconds) == KFormat::ShowMilliseconds) {
-            //: @item:intext Duration format hours, minutes, seconds, milliseconds
-            return tr("%1:%2:%3.%4")
-                .arg(hours, 1, 10, QLatin1Char('0'))
-                .arg(minutes, 2, 10, QLatin1Char('0'))
-                .arg(seconds, 2, 10, QLatin1Char('0'))
-                .arg(ms, 3, 10, QLatin1Char('0'));
-        } else { // Default
-            //: @item:intext Duration format hours, minutes, seconds
-            return tr("%1:%2:%3").arg(hours, 1, 10, QLatin1Char('0')).arg(minutes, 2, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0'));
+        if (options & KFormat::HideSeconds) {
+            return formatSingleAbbreviatedDuration(Minutes, minutes);
         }
+
+        //: @item:intext abbreviated amount of minutes and abbreviated amount of seconds
+        return tr("%1 %2").arg(formatSingleAbbreviatedDuration(Minutes, minutes), formatSingleAbbreviatedDuration(Seconds, seconds));
     }
 
-    Q_UNREACHABLE();
-    return QString();
+    if ((options & KFormat::FoldHours) == KFormat::FoldHours && (options & KFormat::ShowMilliseconds) == KFormat::ShowMilliseconds) {
+        //: @item:intext Duration format minutes, seconds and milliseconds
+        return tr("%1:%2.%3").arg(hours * 60 + minutes, 1, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0')).arg(ms, 3, 10, QLatin1Char('0'));
+    } else if ((options & KFormat::FoldHours) == KFormat::FoldHours) {
+        //: @item:intext Duration format minutes and seconds
+        return tr("%1:%2").arg(hours * 60 + minutes, 1, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0'));
+    } else if ((options & KFormat::HideSeconds) == KFormat::HideSeconds) {
+        //: @item:intext Duration format hours and minutes
+        return tr("%1:%2").arg(hours, 1, 10, QLatin1Char('0')).arg(minutes, 2, 10, QLatin1Char('0'));
+    } else if ((options & KFormat::ShowMilliseconds) == KFormat::ShowMilliseconds) {
+        //: @item:intext Duration format hours, minutes, seconds, milliseconds
+        return tr("%1:%2:%3.%4")
+            .arg(hours, 1, 10, QLatin1Char('0'))
+            .arg(minutes, 2, 10, QLatin1Char('0'))
+            .arg(seconds, 2, 10, QLatin1Char('0'))
+            .arg(ms, 3, 10, QLatin1Char('0'));
+    }
+
+    //: @item:intext Duration format hours, minutes, seconds
+    return tr("%1:%2:%3").arg(hours, 1, 10, QLatin1Char('0')).arg(minutes, 2, 10, QLatin1Char('0')).arg(seconds, 2, 10, QLatin1Char('0'));
 }
 
 QString KFormatPrivate::formatDecimalDuration(quint64 msecs, int decimalPlaces) const
@@ -468,6 +501,11 @@ static QString formatSingleDuration(DurationUnits units, int n)
 {
     // NB: n is guaranteed to be non-negative
     switch (units) {
+    case Years:
+        //: @item:intext %n is a whole number
+        //~ singular %n year
+        //~ plural %n years
+        return KFormatPrivate::tr("%n year(s)", nullptr, n);
     case Days:
         //: @item:intext %n is a whole number
         //~ singular %n day
@@ -568,12 +606,31 @@ QString KFormatPrivate::formatRelativeDateTime(const QDateTime &dateTime, QLocal
         const int minutesToNow = secsToNow / 60;
         if (minutesToNow <= 1) {
             return tr("Just now");
-        } else {
-            //: @item:intext %1 is a whole number
-            //~ singular %n minute ago
-            //~ plural %n minutes ago
-            return tr("%n minute(s) ago", nullptr, minutesToNow);
+        } else if (format == QLocale::NarrowFormat) {
+            //: @item:intext %1 is a whole number, abbreviate minute value
+            //~ singular %n min ago
+            //~ plural %n mins ago
+            return tr("%n min(s) ago", nullptr, minutesToNow);
         }
+        //: @item:intext %1 is a whole number
+        //~ singular %n minute ago
+        //~ plural %n minutes ago
+        return tr("%n minute(s) ago", nullptr, minutesToNow);
+    }
+    if (secsToNow <= 0 && -secsToNow < secsInAHour) {
+        const int minutesFromNow = -secsToNow / 60;
+        if (minutesFromNow < 1) {
+            return tr("Now");
+        } else if (format == QLocale::NarrowFormat) {
+            //: @item:intext %1 is a whole number, abbreviate minute value
+            //~ singular %n min
+            //~ plural %n mins
+            return tr("%n min(s)", nullptr, minutesFromNow);
+        }
+        //: @item:intext %1 is a whole number
+        //~ singular In %n minute
+        //~ plural In %n minutes
+        return tr("In %n minute(s)", nullptr, minutesFromNow);
     }
 
     const auto timeFormatType = format == QLocale::FormatType::LongFormat ? QLocale::FormatType::ShortFormat : format;
@@ -593,6 +650,36 @@ QString KFormatPrivate::formatRelativeDateTime(const QDateTime &dateTime, QLocal
     return formattedDate.replace(0, 1, formattedDate.at(0).toUpper());
 }
 
+[[nodiscard]] static bool needsTimeZone(const QDateTime &dt)
+{
+    if (dt.timeSpec() == Qt::LocalTime) {
+        return false;
+    }
+    return dt.timeZone().offsetFromUtc(dt) != QTimeZone::systemTimeZone().offsetFromUtc(dt);
+}
+
+QString KFormatPrivate::formatTime(const QDateTime &dateTime, QLocale::FormatType format, KFormat::TimeFormatOptions options) const
+{
+    auto output = m_locale.toString(dateTime.time(), format);
+    if (options == KFormat::DoNotAddTimeZone || dateTime.timeSpec() == Qt::LocalTime
+        || ((options & KFormat::AddTimezoneAbbreviationIfNeeded) && !needsTimeZone(dateTime))) {
+        return output;
+    }
+
+    QString tzAbbr;
+    const auto tz = dateTime.timeZone();
+    if (tz.hasDaylightTime()) {
+        tzAbbr = tz.displayName(tz.isDaylightTime(dateTime) ? QTimeZone::DaylightTime : QTimeZone::StandardTime, QTimeZone::ShortName, m_locale);
+    } else {
+        tzAbbr = tz.displayName(QTimeZone::GenericTime, QTimeZone::ShortName, m_locale);
+    }
+    if (tzAbbr.isEmpty()) {
+        return output;
+    }
+    /*: %1 is a formatted time (from QLocale.toString), %2 is a localized timezone abbreviation (from QTimeZone::displayName(QTimeZone::ShortName)). */
+    return tr("%1 %2").arg(output, tzAbbr);
+}
+
 QString KFormatPrivate::formatDistance(double distance, KFormat::DistanceFormatOptions options) const
 {
     if (((options & KFormat::MetricDistanceUnits) == 0)
